"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultRewriteModulePathOptions = void 0;
var path_1 = __importDefault(require("path"));
function join() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    // windows separator
    if (path_1.default.sep === '\\') {
        return path_1.default.join.apply(path_1.default, args).replace(/\\/g, '/');
    }
    return path_1.default.join.apply(path_1.default, args);
}
exports.defaultRewriteModulePathOptions = {
    transform: {
        importDeclaration: true,
        exportDeclaration: true,
        dynamicImport: true,
        importGlob: true,
        importGlobEager: true,
        require: true,
        requireContext: true,
    }
};
function babelPluginRewriteModulePath(_, options) {
    var _a = __assign(__assign({}, exports.defaultRewriteModulePathOptions), options), rewriteMapper = _a.rewriteMapper, transform = _a.transform;
    var rewriters = Object.entries(rewriteMapper).map(function (_a) {
        var from = _a[0], to = _a[1];
        var fromPath = new RegExp(join(from, '/'));
        var toPath = join(to, '/');
        return function (toRewrite) {
            var formattedToRewrite = join(toRewrite);
            if (!fromPath.test(formattedToRewrite)) {
                return null;
            }
            return formattedToRewrite.replace(fromPath, toPath);
        };
    });
    var rewriteStringLiteral = function (toRewrite) {
        var _a;
        return (_a = rewriters.reduce(function (result, rewriter) { return result !== null && result !== void 0 ? result : rewriter(toRewrite); }, null)) !== null && _a !== void 0 ? _a : toRewrite;
    };
    return {
        name: 'rewrite-node-modules',
        visitor: {
            ImportDeclaration: function (path) {
                if (
                /** @param: enable import declaration */ (transform.importDeclaration)
                /**
                 * @example matched cases:
                 * ```
                 * import foo from '@monorepo/shared/modules/foo'
                 * import { bar } from '@monorepo/shared/modules/foo'
                 * import '@monorepo/shared/modules/foo'
                 * ```
                 */
                ) {
                    path.node.source.value = rewriteStringLiteral(path.node.source.value);
                }
            },
            ExportDeclaration: function (path) {
                if (
                /** @param: enable export declaration */ (transform.exportDeclaration)
                    /**
                     * @example matched cases:
                     * ```
                     * export * from '@monorepo/shared/modules/foo'
                     * export { bar } from '@monorepo/shared/modules/foo'
                     * export default from '@monorepo/shared/modules/foo'
                     * ```
                     */
                    &&
                        ('source' in path.node && path.node.source)) {
                    path.node.source.value = rewriteStringLiteral(path.node.source.value);
                }
            },
            CallExpression: function (path) {
                if ((
                /** @param: enable dynamic importing */ (transform.dynamicImport) &&
                    /**
                     * @example matched cases:
                     * ```
                     * const foo = await import('@monorepo/shared/modules/foo')
                     * const { bar } = await import('@monorepo/shared/modules/foo')
                     * import('@monorepo/shared/modules/foo')
                     * ```
                     */
                    (path.node.callee.type === 'Import')) || (
                /** @param: enable require call expression */ (transform.require) &&
                    /**
                     * @example matched cases:
                     * ```
                     * const foo = require('@monorepo/shared/modules/foo')
                     * ```
                     */
                    (path.node.callee.type === 'Identifier' && path.node.callee.name === 'require' &&
                        !path.scope.hasOwnBinding('require'))) || (
                /** @param: enable require.context call expression */ (transform.requireContext) &&
                    /**
                     * @example matched cases:
                     * ```
                     * const foo = require.context('@monorepo/shared/modules/foo')
                     * ```
                     */
                    (path.node.callee.type === 'MemberExpression' && path.node.callee.property.type === 'Identifier' && path.node.callee.property.name === 'context' &&
                        path.node.callee.object.type === 'Identifier' && path.node.callee.object.name === 'require' &&
                        !path.scope.hasOwnBinding('context') && !path.parentPath.scope.hasOwnBinding('require'))) || (
                /** @param: enable glob import call expression */ (transform.importGlob || transform.importGlobEager) &&
                    /**
                     * @example matched cases:
                     * ```
                     * const foo = import.meta.glob('@monorepo/shared/modules/foo')
                     * const foo = import.meta.globEager('@monorepo/shared/modules/foo')
                     * ```
                     */
                    (path.node.callee.type === 'MemberExpression' &&
                        path.node.callee.object.type === 'MetaProperty' && path.node.callee.object.property.type === 'Identifier' &&
                        path.node.callee.object.property.name === 'meta' && path.node.callee.property.type === 'Identifier' &&
                        (transform.importGlob && path.node.callee.property.name === 'glob' && !path.scope.hasOwnBinding('glob') ||
                            transform.importGlobEager && path.node.callee.property.name === 'globEager' && !path.scope.hasOwnBinding('globEager'))))) {
                    var maybeImportStringLiteral = path.node.arguments[0];
                    if (maybeImportStringLiteral.type === 'StringLiteral') {
                        maybeImportStringLiteral.value = rewriteStringLiteral(maybeImportStringLiteral.value);
                    }
                }
            }
        }
    };
}
exports.default = babelPluginRewriteModulePath;
